@@ -1,7 +1,6 @@
 #################################################################################
 ##
-##   R package rugarch by Alexios Ghalanos Copyright (C) 2008, 2009, 2010, 2011, 
-##	 2012
+##   R package rugarch by Alexios Ghalanos Copyright (C) 2008-2013.
 ##   This file is part of the R package rugarch.
 ##
 ##   The R package rugarch is free software: you can redistribute it and/or modify
@@ -27,68 +26,81 @@
 # the Full model does (but is more expensive since we need to simulate the parameter distribution)
 
 
-.ugarchbootfit = function(fitORspec, data = NULL, method = c("Partial", "Full"), n.ahead = 10, n.bootfit = 100, 
-		n.bootpred = 500, out.sample = 0, rseed = NA, solver = "solnp", solver.control = list(), fit.control = list(), 
-		external.forecasts =  list(mregfor = NULL, vregfor = NULL), mexsimdata = NULL, vexsimdata = NULL, parallel = FALSE, 
-		parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2))
+.ugarchbootfit = function(fitORspec, data = NULL, method = c("Partial", "Full"),
+		sampling = c("raw", "kernel", "spd"), 
+		spd.options = list(upper = 0.9, lower = 0.1, type = "pwm", kernel = "normal"),
+		n.ahead = 10, n.bootfit = 100, n.bootpred = 500, out.sample = 0, 
+		rseed = NA, solver = "solnp", solver.control = list(), fit.control = list(), 
+		external.forecasts =  list(mregfor = NULL, vregfor = NULL), 
+		mexsimdata = NULL, vexsimdata = NULL, cluster = NULL, verbose = FALSE)
 {
-
+	if(tolower(sampling[1]) == "kernel") if(!require(ks)) stop("\nPackage 'ks' must be installed") 
+	if(tolower(sampling[1]) == "spd") if(!require(spd)) stop("\nPackage 'spd' must be installed") 
 	method = tolower(method)
 	ans = switch(method,
-			partial = .ub1p1(fitORspec, data = data, n.ahead = n.ahead, n.bootfit = n.bootfit, 
-						n.bootpred = n.bootpred, rseed = rseed, solver.control = solver.control, 
-						fit.control = fit.control, external.forecasts =  external.forecasts, 
-						mexsimdata = mexsimdata, vexsimdata = vexsimdata,
-						parallel = parallel, parallel.control = parallel.control),
-			full = .ub1f1(fitORspec, data = data, n.ahead = n.ahead, n.bootfit = n.bootfit, 
-					n.bootpred = n.bootpred, rseed = rseed, solver = solver, solver.control = solver.control, 
-					fit.control = fit.control, external.forecasts =  external.forecasts, 
-					mexsimdata = mexsimdata, vexsimdata = vexsimdata,
-					parallel = parallel, parallel.control = parallel.control))
+			partial = .ub1p1(fitORspec, data = data, sampling = sampling, spd.options = spd.options,
+					n.ahead = n.ahead, n.bootfit = n.bootfit, n.bootpred = n.bootpred, rseed = rseed, 
+					solver.control = solver.control, fit.control = fit.control, 
+					external.forecasts =  external.forecasts, mexsimdata = mexsimdata, 
+					vexsimdata = vexsimdata, cluster = cluster, verbose = verbose),
+			full = .ub1f1(fitORspec, data = data, sampling = sampling, spd.options = spd.options, 
+					n.ahead = n.ahead, n.bootfit = n.bootfit, n.bootpred = n.bootpred, rseed = rseed, 
+					solver = solver, solver.control = solver.control, fit.control = fit.control, 
+					external.forecasts =  external.forecasts, mexsimdata = mexsimdata, 
+					vexsimdata = vexsimdata, cluster = cluster, verbose = verbose))
 	return(ans)
 }
 
-.ugarchbootspec = function(fitORspec, data = NULL, method = c("Partial", "Full"), n.ahead = 10, n.bootfit = 100, 
-		n.bootpred = 500, out.sample = 0, rseed = NA, solver = "solnp", solver.control = list(), fit.control = list(), 
-		external.forecasts =  list(mregfor = NULL, vregfor = NULL), mexsimdata = NULL, vexsimdata = NULL, 
-		parallel = FALSE, parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2))
+.ugarchbootspec = function(fitORspec, data = NULL, method = c("Partial", "Full"), 
+		sampling = c("raw", "kernel", "spd"), 
+		spd.options = list(upper = 0.9, lower = 0.1, type = "pwm", kernel = "normal"),
+		n.ahead = 10, n.bootfit = 100, n.bootpred = 500, out.sample = 0, rseed = NA, 
+		solver = "solnp", solver.control = list(), fit.control = list(), 
+		external.forecasts =  list(mregfor = NULL, vregfor = NULL), 
+		mexsimdata = NULL, vexsimdata = NULL, cluster = NULL, verbose = FALSE)
 {
 	method = tolower(method)
 	ans = switch(method,
-			partial = .ub1p2(fitORspec, data = data, n.ahead = n.ahead, n.bootfit = n.bootfit, 
-					n.bootpred = n.bootpred, out.sample = out.sample, rseed = rseed, solver.control = solver.control, 
+			partial = .ub1p2(fitORspec, data = data, sampling = sampling, spd.options = spd.options, 
+					n.ahead = n.ahead, n.bootfit = n.bootfit, n.bootpred = n.bootpred, 
+					out.sample = out.sample, rseed = rseed, solver.control = solver.control, 
 					fit.control = fit.control, external.forecasts =  external.forecasts, 
 					mexsimdata = mexsimdata, vexsimdata = vexsimdata,
-					parallel = parallel, parallel.control = parallel.control),
-			full = .ub1f2(fitORspec, data = data, n.ahead = n.ahead, n.bootfit = n.bootfit, 
-					n.bootpred = n.bootpred, out.sample = out.sample, rseed = rseed, solver = solver, 
-					solver.control = solver.control, fit.control = fit.control, external.forecasts =  external.forecasts, 
-					mexsimdata = mexsimdata, vexsimdata = vexsimdata,
-					parallel = parallel, parallel.control = parallel.control))
+					cluster = cluster, verbose = verbose),
+			full = .ub1f2(fitORspec, data = data, sampling = sampling, spd.options = spd.options, 
+					n.ahead = n.ahead, n.bootfit = n.bootfit, n.bootpred = n.bootpred, 
+					out.sample = out.sample, rseed = rseed, solver = solver, 
+					solver.control = solver.control, fit.control = fit.control, 
+					external.forecasts =  external.forecasts, mexsimdata = mexsimdata, 
+					vexsimdata = vexsimdata, cluster = cluster, verbose = verbose))
 	return(ans)
 }
 
 # method from a fit object
-.ub1f1 = function(fitORspec, data = NULL, n.ahead = 10, n.bootfit = 100, n.bootpred = 500, rseed = NA, solver = "solnp",
-		solver.control = list(), fit.control = list(), external.forecasts =  list(mregfor = NULL, vregfor = NULL), 
-		mexsimdata = NULL, vexsimdata = NULL, parallel = FALSE, 
-		parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2))
+.ub1f1 = function(fitORspec, data = NULL, sampling = c("raw", "kernel", "spd"), 
+		spd.options = list(upper = 0.9, lower = 0.1, type = "pwm", kernel = "normal"), 
+		n.ahead = 10, n.bootfit = 100, n.bootpred = 500, rseed = NA, solver = "solnp", 
+		solver.control = list(), fit.control = list(), 
+		external.forecasts =  list(mregfor = NULL, vregfor = NULL), 
+		mexsimdata = NULL, vexsimdata = NULL, cluster = NULL, verbose = FALSE)
 {
 	
 	# inputs:
 	# n.bootfit		: the number of simulations for which we will generate the parameter 
 	#				uncertainty distribution (i.e. no of refits)
-	# n.bootpred	: the number of simulations / h.ahead to use (for averaging to obtain forecast
+	# n.bootpred	: the number of simulations / n.ahead to use (for averaging to obtain forecast
 	#				density)
 	# out.sample	: data to keep for out of sample comparison purposes
 	# n.ahead		: the horizon of the bootstrap density forecast
+	sampling = tolower(sampling[1])
+	if(is.na(match(sampling, c("raw", "kernel", "spd")))) stop("\nsampling method not recognized")
 	fit = fitORspec
 	model = fit@model
 	m = model$maxOrder
 	data = model$modeldata$data
 	N = model$modeldata$T
 	ns = model$n.start
-	if (is.na(rseed[1])){
+	if(is.na(rseed[1])){
 		sseed = as.integer(runif(n.bootpred + n.bootfit, 0, 65000))
 	} else{
 		if(length(rseed) < n.bootpred){
@@ -102,12 +114,17 @@
 	# generate paths of equal length to data based on empirical re-sampling of z
 	# p.2296 equation (5)
 	fz = fit@fit$z
-	empz = matrix(0, ncol = n.bootfit, nrow = N)
+	if(sampling!="raw"){
+		zfit = .bfit(fz, sampling, spd.options)
+	} else{
+		zfit = NULL
+	}
 	
+	empz = matrix(0, ncol = n.bootfit, nrow = N)
 	empz = apply(as.data.frame(1:n.bootfit), 1, FUN=function(i) {
 				set.seed(sseed1[i]);
-				sample(fz, size = N, replace = TRUE);})
-		
+				.bsample(fz, N, sampling, zfit);})	
+	
 	if(ns > 0) {
 		spec = getspec(fit)
 		setfixed(spec) <- as.list(coef(fit))
@@ -121,9 +138,12 @@
 	# presigma uses the same starting values as the original fit
 	# -> in paper they use alternatively the unconditional long run sigma (P.2296 paragraph 2
 	# "...marginal variance..."
-	paths = ugarchsim(fit, n.sim = N, m.sim = n.bootfit, presigma = tail(fit@fit$sigma, m), 
-			prereturns = tail(model$modeldata$data[1:N], m), preresiduals = tail(residuals(fit), m), 
-			startMethod = "sample", custom.dist = list(name = "sample", distfit = as.matrix(empz)),
+	paths = ugarchsim(fit, n.sim = N, m.sim = n.bootfit, 
+			presigma = tail(fit@fit$sigma, m), 
+			prereturns = tail(model$modeldata$data[1:N], m), 
+			preresiduals = tail(residuals(fit), m), 
+			startMethod = "sample", 
+			custom.dist = list(name = "sample", distfit = as.matrix(empz)),
 			rseed = sseed1, mexsimdata = mexsimdata, vexsimdata = vexsimdata)
 	fitlist = vector(mode="list", length = n.bootfit)
 	path.df = as.data.frame(paths, which = "series")
@@ -135,52 +155,33 @@
 	
 	# get the distribution of the parameters (by fitting to the new path data)
 	#-------------------------------------------------------------------------
-	cat("\nfitting stage...")
-	if( parallel ){
-		os = .Platform$OS.type
-		if(is.null(parallel.control$pkg)){
-			if( os == "windows" ) parallel.control$pkg = "snowfall" else parallel.control$pkg = "multicore"
-			if( is.null(parallel.control$cores) ) parallel.control$cores = 2
-		} else{
-			mtype = match(tolower(parallel.control$pkg[1]), c("multicore", "snowfall"))
-			if(is.na(mtype)) stop("\nParallel Package type not recognized in parallel.control\n")
-			parallel.control$pkg = tolower(parallel.control$pkg[1])
-			if( os == "windows" && parallel.control$pkg == "multicore" ) stop("\nmulticore not supported on windows O/S\n")
-			if( is.null(parallel.control$cores) ) parallel.control$cores = 2 else parallel.control$cores = as.integer(parallel.control$cores[1])
-		}
-		if( parallel.control$pkg == "multicore" ){
-			if(!exists("mclapply")){
-				require('multicore')
-			}
-			fitlist = multicore::mclapply(1:nx, FUN = function(i) ugarchfit(spec = spec, data = path.df[, i, drop = FALSE], out.sample = 0,
-						solver = solver, fit.control = fit.control, solver.control = solver.control), mc.cores = parallel.control$cores)
-		} else{
-			if(!exists("sfLapply")){
-				require('snowfall')
-			}
-			nx = dim(path.df)[2]
-			sfInit(parallel=TRUE, cpus = parallel.control$cores)
-			sfExport("path.df", "spec", "solver", "out.sample", "solver.control", "fit.control", local = TRUE)
-			fitlist = sfLapply(as.list(1:nx), fun = function(i)
-						rugarch::ugarchfit(spec = spec, data = path.df[, i, drop = FALSE], out.sample = 0,
-								solver = solver, fit.control = fit.control, solver.control = solver.control))
-			#sfStop()
-		}
+	if( verbose ) cat("\nfitting stage...")
+	if( !is.null(cluster) ){
+		nx = dim(path.df)[2]
+		parallel::clusterEvalQ(cluster, library(rugarch))
+		parallel::clusterExport(cluster, c("path.df", "spec", "solver", 
+						"solver.control", "fit.control"), envir = environment())
+		fitlist = parallel::parLapply(cluster, as.list(1:nx), fun = function(i){
+					rugarch:::.safefit(spec = spec, data = path.df[, i, drop = FALSE], 
+							out.sample = 0, solver = solver, fit.control = fit.control, 
+							solver.control = solver.control)
+				})
 	} else{
-		for(i in 1:n.bootfit){
-			fitlist[[i]] = .safefit(spec = spec, data = path.df[, i, drop = FALSE], out.sample = 0, fit.control = fit.control, 
-					solver = solver, solver.control = solver.control)
-		}
+		fitlist = lapply(as.list(1:nx), fun = function(i){
+					.safefit(spec = spec, data = path.df[, i, drop = FALSE], 
+							out.sample = 0, solver = solver, fit.control = fit.control, 
+							solver.control = solver.control)
+				})
 	}
 	# check for any non convergence and remove
-	if(any(sapply(fitlist,FUN=function(x) is.null(coef(x))))){
+	if(any(sapply(fitlist, FUN=function(x) is.null(coef(x))))){
 		exc = which(sapply(fitlist,FUN=function(x) is.null(coef(x))))
 		fitlist = fitlist[-exc]
 		n.bootfit = n.bootfit - length(exc)
 		# in case something went very wrong:
 		if(n.bootfit == 0) stop("\nugarchboot-->error: the fitting routine failed. No convergene at all!\n", call. = FALSE)
 	}
-	cat("done!\n")
+	if( verbose ) cat("done!\n")
 	# extract the coefficient distribution and generate spec list to feed to path function
 	coefdist = matrix(NA, ncol = length(coef(fit)), nrow = n.bootfit)
 	speclist = vector(mode = "list", length = n.bootfit)
@@ -191,7 +192,6 @@
 	}
 	colnames(coefdist) = names(coef(fit))
 	#-------------------------------------------------------------------------
-
 	# generate path based forecast values
 	# for each path we generate n.bootpred vectors of resampled data of length n.ahead
 	# Equation (6) in the PRR paper (again using z from original fit)
@@ -201,66 +201,46 @@
 		empz = matrix(0, ncol = n.bootpred, nrow = n.ahead)
 		empz = apply(as.data.frame(1:n.bootpred), 1, FUN=function(i) {
 					set.seed(sseed2[i]);
-					sample(fz, size = n.ahead, replace = TRUE);})
+					.bsample(fz, n.ahead, sampling, zfit);})
 		empzlist[[i]] = matrix(empz, ncol = n.bootpred)
 	}
 	# we start all forecasts from the last value of sigma based on the original series but
 	# the pertrubed parameters as in equation (7) in PRR paper.
-	if( parallel ){
-		if( parallel.control$pkg == "multicore" ){
-			st = multicore::mclapply(as.list(1:n.bootfit), FUN = function(x) .sigmat(spec = speclist[[i]], origdata = data[1:N], m), mc.cores = parallel.control$cores)
-			st = matrix(sapply(st, FUN = function(x) x), ncol = n.bootfit)
-		} else{
-			nx = length(speclist)
-			#sfInit(parallel = TRUE, cpus = parallel.control$cores)
-			sfExport("speclist", "data", "m", "N", local = TRUE)
-			st = sfLapply(as.list(1:n.bootfit), fun = function(i) rugarch:::.sigmat(spec = speclist[[i]], origdata =  data[1:N], m))
-			#sfStop()
-			st = matrix(sapply(st, FUN = function(x) x), ncol = n.bootfit)
-		}
+	if( !is.null(cluster) ){
+		nx = length(speclist)
+		parallel::clusterExport(cluster, c("speclist", "data", "m", "N"), envir = environment())
+		st = parallel::parLapply(cluster, as.list(1:n.bootfit), fun = function(i){
+					rugarch:::.sigmat(spec = speclist[[i]], origdata =  data[1:N], m)
+				})
+		st = matrix(sapply(st, FUN = function(x) x), ncol = n.bootfit)
 	} else{
 		st = matrix(sapply(speclist, FUN=function(x) .sigmat(spec = x, origdata =  data[1:N], m)), ncol = n.bootfit)
 	}
 	forcseries = NULL
 	forcsigma  = NULL
 	tmp = vector(mode = "list", length = n.bootfit)
-	cat("\nprediction stage...")
+	if( verbose ) cat("\nprediction stage...")
 	xdat = tail( data[1:N], m )
-	if( parallel ){
-		if( parallel.control$pkg == "multicore" ){
-		tmp = multicore::mclapply(as.list(1:n.bootfit), FUN = function(i) .quicksimulate(fitlist[[i]], n.sim = n.ahead, 
+	if( !is.null(cluster) ){
+		parallel::clusterExport(cluster, c("fitlist", "n.ahead", "n.bootpred", "n.bootfit", 
+						"st", "xdat", "sseed", "empzlist","mexsimdata", "vexsimdata"), 
+				envir = environment())
+		tmp = parallel::parLapply(cluster, as.list(1:n.bootfit), fun = function(i){
+					rugarch:::.quicksimulate(fitlist[[i]], n.sim = n.ahead, 
 							m.sim = n.bootpred, presigma = st[,i], prereturns = xdat,  
 							n.start = 0, rseed = sseed[(n.bootfit+1):(n.bootfit + n.bootpred)], 
 							custom.dist = list(name = "sample", distfit = as.matrix(empzlist[[i]])), 
-							mexsimdata = mexsimdata, vexsimdata = vexsimdata), 
-				mc.cores = parallel.control$cores)
-		} else{
-			#sfInit(parallel = TRUE, cpus = parallel.control$cores)
-			sfExport("fitlist", "n.ahead", "n.bootpred", "n.bootfit", "st", "xdat", "sseed", "empzlist",
-					"mexsimdata", "vexsimdata", local = TRUE)
-			tmp = sfLapply(as.list(1:n.bootfit), fun = function(i) rugarch:::.quicksimulate(fitlist[[i]], n.sim = n.ahead, 
-								m.sim = n.bootpred, presigma = st[,i], prereturns = xdat,  
-								n.start = 0, rseed = sseed[(n.bootfit+1):(n.bootfit + n.bootpred)], 
-								custom.dist = list(name = "sample", distfit = as.matrix(empzlist[[i]])), 
-								mexsimdata = mexsimdata, vexsimdata = vexsimdata))
-			sfStop()
-		}
-	} else{
-		tmp = vector(mode = "list", length = n.bootfit)
-		for(i in 1:n.bootfit){
-			tmp[[i]] = .quicksimulate(fit = fitlist[[i]], n.sim = n.ahead, 
+							mexsimdata = mexsimdata, vexsimdata = vexsimdata)
+				})
+	} else{	
+		tmp = lapply(as.list(1:n.bootfit), FUN = function(i){
+					.quicksimulate(fit = fitlist[[i]], n.sim = n.ahead, 
 					m.sim = n.bootpred, presigma = st[,i], prereturns = xdat,  
 					n.start = 0, rseed = sseed[(n.bootfit+1):(n.bootfit + n.bootpred)], 
 					custom.dist = list(name = "sample", distfit = as.matrix(empzlist[[i]])), 
 					mexsimdata = mexsimdata, vexsimdata = vexsimdata)
-			# reduce memory
-			empzlist[[i]] = 0
-			gc(verbose = FALSE)
-		}
+		})
 	}
-	# do some cleanup
-	rm(empzlist)
-	gc(verbose = FALSE)
 
 	# we will have (n.bootfit x n.bootpred) x n.ahead matrix
 	forcseries = lapply(tmp, FUN = function(x) x[1:n.ahead, ,drop = F])
@@ -273,8 +253,10 @@
 	forcsigma = t(forcsigma)
 	# do some cleanup
 	rm(tmp)
+	rm(fitlist)
+	rm(empzlist)
 	gc(verbose = FALSE)	
-	cat("done!\n")
+	if( verbose ) cat("done!\n")
 	#-------------------------------------------------------------------------
 
 	# now we have the bootstrapped distribution of n.ahead forecast values
@@ -290,6 +272,7 @@
 	model$modeldata$meanfit.s = meansigmafit
 	model$seeds = sseed
 	model$type = "full"
+	model$dateT = model$modeldata$dates[N]
 	ans = new("uGARCHboot",
 			fseries = forcseries,
 			fsigma = forcsigma,
@@ -301,12 +284,15 @@
 
 
 # method from a spec object
-.ub1f2 = function(fitORspec, data = NULL, n.ahead = 10, n.bootfit = 100, 
-		n.bootpred = 500, out.sample = 0, rseed = NA, solver = "solnp", solver.control = list(), fit.control = list(), 
-		external.forecasts =  list(mregfor = NULL, vregfor = NULL), mexsimdata = NULL, vexsimdata = NULL, parallel = FALSE, 
-		parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2))
+.ub1f2 = function(fitORspec, data = NULL, sampling = c("raw", "kernel", "spd"), 
+		spd.options = list(upper = 0.9, lower = 0.1, type = "pwm", kernel = "normal"), 
+		n.ahead = 10, n.bootfit = 100, n.bootpred = 500, out.sample = 0, rseed = NA, 
+		solver = "solnp", solver.control = list(), fit.control = list(),  
+		external.forecasts =  list(mregfor = NULL, vregfor = NULL), 
+		mexsimdata = NULL, vexsimdata = NULL, cluster = NULL, verbose = FALSE)
 {
-
+	sampling = tolower(sampling[1])
+	if(is.na(match(sampling, c("raw", "kernel", "spd")))) stop("\nsampling method not recognized")
 	spec = fitORspec
 	model = spec@model
 	if(is.null(data))
@@ -342,11 +328,20 @@
 	# p.2296 equation (5)
 	# use of filter when using spec/this will also check whether the fixed.pars are correctly
 	# specified
+	# ::length(data)-out.sample
 	fz = flt@filter$z
+	
+	empz = matrix(0, ncol = n.bootfit, nrow = N)
+	if(sampling!="raw"){
+		zfit = .bfit(fz, sampling, spd.options)
+	} else{
+		zfit = NULL
+	}
+	
 	empz = matrix(0, ncol = n.bootfit, nrow = N)
 	empz = apply(as.data.frame(1:n.bootfit), 1, FUN=function(i) {
 				set.seed(sseed1[i]);
-				sample(fz, size = N, replace = TRUE);})
+				.bsample(fz, N, sampling, zfit);})	
 	# presigma uses the same starting values as the original fit
 	# -> in paper they use alternatively the unconditional long run sigma (P.2296 paragraph 2
 	# "...marginal variance...")
@@ -365,42 +360,23 @@
 	setstart(spex) <- spec@model$fixed.pars
 	# get the distribution of the parameters (by fitting to the new path data)
 	#-------------------------------------------------------------------------
-	cat("\nfitting stage...")
-	if( parallel ){
-		os = .Platform$OS.type
-		if(is.null(parallel.control$pkg)){
-			if( os == "windows" ) parallel.control$pkg = "snowfall" else parallel.control$pkg = "multicore"
-			if( is.null(parallel.control$cores) ) parallel.control$cores = 2
-		} else{
-			mtype = match(tolower(parallel.control$pkg[1]), c("multicore", "snowfall"))
-			if(is.na(mtype)) stop("\nParallel Package type not recognized in parallel.control\n")
-			parallel.control$pkg = tolower(parallel.control$pkg[1])
-			if( os == "windows" && parallel.control$pkg == "multicore" ) stop("\nmulticore not supported on windows O/S\n")
-			if( is.null(parallel.control$cores) ) parallel.control$cores = 2 else parallel.control$cores = as.integer(parallel.control$cores[1])
-		}
-		if( parallel.control$pkg == "multicore" ){
-			if(!exists("mclapply")){
-				require('multicore')
-			}
-			fitlist = multicore::mclapply(as.list(1:nx), FUN = function(x) ugarchfit(spec = spex, data = path.df[,i,drop = FALSE], out.sample = 0,
-								solver = solver, fit.control = fit.control, solver.control = solver.control), mc.cores = parallel.control$cores)
-		} else{
-			if(!exists("sfLapply")){
-				library('snowfall', pos = "package:base")
-			}
+	if( verbose ) cat("\nfitting stage...")
+	if( !is.null(cluster) ){
+			parallel::clusterEvalQ(cluster, library(rugarch))
 			nx = dim(path.df)[2]
-			sfInit(parallel=TRUE, cpus = parallel.control$cores)
-			sfExport("path.df", "spex", "solver", "out.sample", "solver.control", "fit.control", local = TRUE)
-			fitlist = sfLapply(as.list(1:nx), fun = function(i)
-						rugarch::ugarchfit(spec = spex, data = path.df[,i,drop = FALSE], out.sample = 0,
-								solver = solver, fit.control = fit.control, solver.control = solver.control))
-			#sfStop()
-		}
+			parallel::clusterExport(cluster, c("path.df", "spex", "solver", "out.sample", 
+							"solver.control", "fit.control"), envir = environment())
+			fitlist = parallel::parLapply(cluster, as.list(1:nx), fun = function(i){
+						rugarch:::.safefit(spec = spex, data = path.df[,i,drop = FALSE], 
+								out.sample = 0, solver = solver, fit.control = fit.control, 
+								solver.control = solver.control)
+					})
 	} else{
-		for(i in 1:n.bootfit){
-			fitlist[[i]] = .safefit(spec = spex, data = path.df[,i,drop = FALSE], out.sample = 0, fit.control = fit.control, 
-					solver = solver, solver.control = solver.control)
-		}
+		fitlist = lapply(as.list(1:nx), FUN = function(i){
+					.safefit(spec = spex, data = path.df[,i,drop = FALSE], 
+							out.sample = 0, solver = solver, fit.control = fit.control, 
+							solver.control = solver.control)
+				})
 	}
 	# check for any non convergence and remove
 	if(any(sapply(fitlist,FUN=function(x) is.null(coef(x))))){
@@ -410,7 +386,7 @@
 		# in case something went very wrong:
 		if(n.bootfit == 0) stop("\nugarchboot-->error: the fitting routine failed. No convergene at all!\n", call. = FALSE)
 	}
-	cat("done!\n")
+	if( verbose ) cat("done!\n")
 	
 	# extract the coefficient distribution and generate spec list to feed to path function
 	coefdist = matrix(NA, ncol = length(coef(flt)), nrow = n.bootfit)
@@ -431,24 +407,19 @@
 		empz = matrix(0, ncol = n.bootpred, nrow = n.ahead)
 		empz = apply(as.data.frame(1:n.bootpred), 1, FUN=function(i) {
 					set.seed(sseed2[i]);
-					sample(fz, size = n.ahead, replace = TRUE);})
+					.bsample(fz, n.ahead, sampling, zfit);})
 		empzlist[[i]] = matrix(empz, ncol = n.bootpred)
 	}
 	
 	# we start all forecasts from the last value of sigma based on the original series but
 	# the pertrubed parameters as in equation (7) in PRR paper.
-	if( parallel ){
-		if( parallel.control$pkg == "multicore" ){
-			st = multicore::mclapply(speclist, FUN=function(x) .sigmat(spec = x, origdata = xdata[1:N], m), mc.cores = parallel.control$cores)
-			st = matrix(sapply(st, FUN = function(x) x), ncol = n.bootfit)
-		} else{
+	if( !is.null(cluster) ){
 			nx = length(speclist)
-			#sfInit(parallel = TRUE, cpus = parallel.control$cores)
-			sfExport("speclist", "xdata", "m", "N", local = TRUE)
-			st = sfLapply(as.list(1:nx), fun = function(i) rugarch:::.sigmat(spec = speclist[[i]], origdata = xdata[1:N], m))
-			sfStop()
+			parallel::clusterExport(cluster, c("speclist", "xdata", "m", "N"), envir = environment())
+			st = parallel::parLapply(cluster, as.list(1:nx), fun = function(i){
+						rugarch:::.sigmat(spec = speclist[[i]], origdata = xdata[1:N], m)
+					})
 			st = matrix(sapply(st, FUN = function(x) x), ncol = n.bootfit)
-		}
 	} else{
 		st = matrix(sapply(speclist, FUN=function(x) .sigmat(spec = x, origdata =  xdata[1:N], m)), ncol = n.bootfit)		
 	}
@@ -458,39 +429,28 @@
 	forcseries = NULL
 	forcsigma  = NULL
 	tmp = vector(mode = "list", length = n.bootfit)
-	cat("\nprediction stage...")
+	if( verbose ) cat("\nprediction stage...")
 	
 	xdat =  tail(xdata[1:N], m)
-	if( parallel ){
-		if( parallel.control$pkg == "multicore" ){			
-			tmp = multicore::mclapply(as.list(1:n.bootfit), FUN = function(i) .quicksimulate(fitlist[[i]], n.sim = n.ahead, 
-								m.sim = n.bootpred, presigma = st[,i], prereturns = xdat,  
-								n.start = 0, rseed = sseed[(n.bootfit+1):(n.bootfit + n.bootpred)], 
-								custom.dist = list(name = "sample", distfit = as.matrix(empzlist[[i]])), 
-								mexsimdata = mexsimdata, vexsimdata = vexsimdata), 
-					mc.cores = parallel.control$cores)
-		} else{
-			#sfInit(parallel = TRUE, cpus = parallel.control$cores)
-			sfExport("fitlist", "n.ahead", "n.bootpred", "n.bootfit", "st", "xdat", "sseed", "empzlist",
-					"mexsimdata", "vexsimdata", local = TRUE)
-			tmp = sfLapply(as.list(1:n.bootfit), fun = function(i) rugarch:::.quicksimulate(fitlist[[i]], n.sim = n.ahead, 
-								m.sim = n.bootpred, presigma = st[,i], prereturns = xdat,  
-								n.start = 0, rseed = sseed[(n.bootfit+1):(n.bootfit + n.bootpred)], 
-								custom.dist = list(name = "sample", distfit = as.matrix(empzlist[[i]])), 
-								mexsimdata = mexsimdata, vexsimdata = vexsimdata))
-			sfStop()
-		}
+	if( !is.null(cluster) ){
+		parallel::clusterExport(cluster, c("fitlist", "n.ahead", "n.bootpred", "n.bootfit", 
+						"st", "xdat", "sseed", "empzlist", "mexsimdata", "vexsimdata"), 
+				envir = environment())
+		tmp = parallel::parLapply(cluster, as.list(1:n.bootfit), fun = function(i){
+					rugarch:::.quicksimulate(fitlist[[i]], n.sim = n.ahead, 
+							m.sim = n.bootpred, presigma = st[,i], prereturns = xdat,  
+							n.start = 0, rseed = sseed[(n.bootfit+1):(n.bootfit + n.bootpred)], 
+							custom.dist = list(name = "sample", distfit = as.matrix(empzlist[[i]])), 
+							mexsimdata = mexsimdata, vexsimdata = vexsimdata)
+				})
 	} else{
-		tmp = vector(mode = "list", length = n.bootfit)
-		for(i in 1:n.bootfit){
-			tmp[[i]] = .quicksimulate(fit = fitlist[[i]], n.sim = n.ahead, 
-					m.sim = n.bootpred, presigma = st[,i], prereturns = xdat,  
-					n.start = 0, rseed = sseed[(n.bootfit+1):(n.bootfit + n.bootpred)], 
-					custom.dist = list(name = "sample", distfit = as.matrix(empzlist[[i]])), 
-					mexsimdata = mexsimdata, vexsimdata = vexsimdata)
-			empzlist[[i]] = 0
-			gc(verbose = FALSE)
-		}
+		tmp = lapply(as.list(1:n.bootfit), FUN = function(i){
+					.quicksimulate(fitlist[[i]], n.sim = n.ahead, m.sim = n.bootpred, 
+							presigma = st[,i], prereturns = xdat,  n.start = 0, 
+							rseed = sseed[(n.bootfit+1):(n.bootfit + n.bootpred)], 
+							custom.dist = list(name = "sample", distfit = as.matrix(empzlist[[i]])), 
+							mexsimdata = mexsimdata, vexsimdata = vexsimdata)
+				})
 	}
 	# we will have (n.bootfit x n.bootpred) x n.ahead matrix
 	forcseries = lapply(tmp, FUN = function(x) x[1:n.ahead, ,drop = F])
@@ -501,11 +461,12 @@
 	meansigmafit = t(sapply(forcsigma, FUN = function(x) apply(t(x), 2, "mean")))
 	forcsigma = matrix(unlist(forcsigma), nrow = n.ahead, ncol = n.bootpred * n.bootfit, byrow = FALSE)
 	forcsigma = t(forcsigma)
-	cat("done!\n")
+	# do some cleanup
 	rm(empzlist)
 	rm(tmp)
 	rm(fitlist)
 	gc(verbose = FALSE)
+	if( verbose ) cat("done!\n")
 	#-------------------------------------------------------------------------
 	# now we have the bootstrapped distribution of n.ahead forecast values
 	# original forecast
@@ -520,6 +481,7 @@
 	model$modeldata$meanfit.s = meansigmafit
 	model$seeds = sseed
 	model$type = "full"
+	model$dateT = flt@model$modeldata$dates[N]
 	ans = new("uGARCHboot",
 			fseries = forcseries,
 			fsigma = forcsigma,
@@ -530,11 +492,15 @@
 }
 
 # Partial method (very fast but does not take into account the parameter uncertainty)
-.ub1p1 = function(fitORspec, data = NULL, n.ahead = 10, n.bootfit = 100, 
-		n.bootpred = 500, rseed = NA, solver.control = list(), fit.control = list(),
-		external.forecasts =  list(mregfor = NULL, vregfor = NULL), mexsimdata = NULL, vexsimdata = NULL, 
-		parallel = FALSE, parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2))
+.ub1p1 = function(fitORspec, data = NULL, sampling = c("raw", "kernel", "spd"), 
+		spd.options = list(upper = 0.9, lower = 0.1, type = "pwm", kernel = "normal"), 
+		n.ahead = 10, n.bootfit = 100, n.bootpred = 500, rseed = NA, 
+		solver.control = list(), fit.control = list(),
+		external.forecasts =  list(mregfor = NULL, vregfor = NULL), 
+		mexsimdata = NULL, vexsimdata = NULL,  cluster = NULL, verbose = FALSE)
 {
+	sampling = tolower(sampling[1])
+	if(is.na(match(sampling, c("raw", "kernel", "spd")))) stop("\nsampling method not recognized")
 	fit = fitORspec
 	model = fit@model
 	m = fit@model$maxOrder
@@ -557,9 +523,18 @@
 	# for each path we generate n.bootpred vectors of resampled data of length n.ahead
 	# Equation (6) in the PRR paper
 	fz = fit@fit$z
-	empz = matrix(apply(as.data.frame(1:n.bootpred), 1, FUN = function(i) {
-														set.seed(sseed[i]);
-														sample(fz, size = n.ahead, replace = TRUE);}), ncol = n.bootpred)
+	
+	if(sampling!="raw"){
+		zfit = .bfit(fz, sampling, spd.options)
+	} else{
+		zfit = NULL
+	}
+	
+	empz = matrix(apply(as.data.frame(1:n.bootpred), 1, 
+					FUN = function(i) {
+						set.seed(sseed[i]);
+						.bsample(fz, n.ahead, sampling, zfit);}), 
+			ncol = n.bootpred)
 	
 	#empz[1:m,] = matrix(rep(tail(fit@fit$z, m), n.bootpred), nrow = m)
 	
@@ -580,7 +555,8 @@
 	sim = vector(mode = "list", length = 1)
 	sim = ugarchsim(fit, n.sim = n.ahead, m.sim = n.bootpred, 
 				presigma = st, prereturns = tail(xdata[1:N], m), preresiduals = tail(residuals(fit), m),
-				rseed = sseed, n.start = 0, startMethod = "sample", custom.dist = list(name = "sample", distfit = empz),
+				rseed = sseed, n.start = 0, startMethod = "sample", 
+				custom.dist = list(name = "sample", distfit = empz),
 				mexsimdata = mexsimdata, vexsimdata = vexsimdata)
 	# we transpose to get n.boot x n.ahead
 	forcseries = t(sim@simulation$seriesSim)
@@ -597,6 +573,7 @@
 	model$n.bootfit = n.bootfit
 	model$n.bootpred = n.bootpred
 	model$type = "partial"
+	model$dateT = model$modeldata$dates[N]
 	ans = new("uGARCHboot",
 			fseries = (forcseries),
 			fsigma = (forcsigma),
@@ -607,11 +584,15 @@
 }
 
 # using spec method
-.ub1p2 = function(fitORspec, data = NULL, n.ahead = 10, n.bootfit = 100, 
-		n.bootpred = 500, out.sample = 0, rseed = NA, solver.control = list(), fit.control = list(), 
-		external.forecasts =  list(mregfor = NULL, vregfor = NULL), mexsimdata = NULL, vexsimdata = NULL, 
-		parallel = FALSE, parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2))
+.ub1p2 = function(fitORspec, data = NULL, sampling = c("raw", "kernel", "spd"), 
+		spd.options = list(upper = 0.9, lower = 0.1, type = "pwm", kernel = "normal"), 
+		n.ahead = 10, n.bootfit = 100, n.bootpred = 500, out.sample = 0, rseed = NA, 
+		solver.control = list(), fit.control = list(), 
+		external.forecasts =  list(mregfor = NULL, vregfor = NULL), 
+		mexsimdata = NULL, vexsimdata = NULL, cluster = NULL, verbose = FALSE)
 {
+	sampling = tolower(sampling[1])
+	if(is.na(match(sampling, c("raw", "kernel", "spd")))) stop("\nsampling method not recognized")
 	spec = fitORspec
 	if(is.null(data))
 		stop("\nugarchboot-->error: data must be supplied if SPEC object used.", call. = FALSE)
@@ -636,11 +617,18 @@
 		}
 	}
 	# generate path based forecast values
-	empz = matrix(0, ncol = n.bootpred, nrow = n.ahead)
 	fz = flt@filter$z
-	empz = matrix(apply(as.data.frame(1:n.bootpred), 1, FUN = function(i) {
+	if(sampling!="raw"){
+		zfit = .bfit(fz, sampling, spd.options)
+	} else{
+		zfit = NULL
+	}
+	
+	empz = matrix(apply(as.data.frame(1:n.bootpred), 1, 
+					FUN = function(i) {
 						set.seed(sseed[i]);
-						sample(fz, size = n.ahead, replace = TRUE);}), ncol = n.bootpred)
+						.bsample(fz, n.ahead, sampling, zfit);}), 
+			ncol = n.bootpred)
 	
 	if(ns > 0) {
 		realized.x = xdata[(N+1):(N+ns)]
@@ -659,7 +647,8 @@
 	sim = ugarchpath(spec, n.sim = n.ahead, m.sim = n.bootpred, presigma = tail(sigma, m),
 				prereturns = tail(xdata[1:N], m), preresiduals = tail(flt@filter$residuals, m),
 				rseed = sseed, n.start = 0, custom.dist = list(name = "sample", 
-						distfit = as.matrix(empz)), mexsimdata = mexsimdata, vexsimdata = vexsimdata)
+						distfit = as.matrix(empz)), mexsimdata = mexsimdata, 
+				vexsimdata = vexsimdata)
 	# we transpose to get n.boot x n.ahead
 	forcseries = t(sim@path$seriesSim)
 	forcsigma =  t(sim@path$sigmaSim)
@@ -676,6 +665,7 @@
 	model$n.bootpred = n.bootpred
 	coefdist = data.frame(NULL)
 	model$type = "partial"
+	model$dateT = flt@model$modeldata$dates[N]
 	ans = new("uGARCHboot",
 			fseries = (forcseries),
 			fsigma = (forcsigma),
@@ -702,4 +692,40 @@
 			vexsimdata = vexsimdata)
 	ret = rbind(ans@simulation$seriesSim, ans@simulation$sigmaSim)
 	return(ret)
+}
+
+.bsample = function(Z, n, method, fit)
+{
+	return(switch(tolower(method),
+			"raw" = sample(Z, n, replace = TRUE),
+			"kernel" = ks::rkde(n, fit),
+			"spd" = spd::rspd(n, fit)))
+}
+
+.bfit = function(Z, method, spd.options){
+	return(switch(tolower(method),
+					"kernel" = .kfit(Z),
+					"spd" = .sfit(Z, spd.options)))
+}
+
+.kfit = function(Z){
+	H.pi = hpi(Z)
+	fit = ks::kde(Z, h=H.pi)
+	return(fit)
+}
+
+.sfit = function(Z, spd.options){
+	mm = match(names(spd.options), c("upper", "lower", "type", "kernel"))
+	if(any(is.na(mm))){
+		idx = which(is.na(mm))
+		enx = NULL
+		for(i in 1:length(idx)) enx = c(enx, names(spd.options)[idx[i]])
+		warning(paste(c("unidentified option(s) in spd.options:\n", enx), sep="", collapse=" "), call. = FALSE, domain = NULL)
+	}
+	if(is.null(spd.options$upper)) upper = 0.9 else upper = spd.options$upper
+	if(is.null(spd.options$upper)) lower = 0.1 else lower = spd.options$lower
+	if(is.null(spd.options$type)) type = "pwm" else type  = spd.options$type[1]
+	if(is.null(spd.options$kernel)) kernel = "normal" else kernel  = spd.options$kernel[1]
+	fit = spd::spdfit(Z, upper = upper, lower = lower, type = type, kernelfit = kernel)
+	return(fit)
 }
\ No newline at end of file
