@@ -1,7 +1,6 @@
 #################################################################################
 ##
-##   R package rugarch by Alexios Ghalanos Copyright (C) 2008, 2009, 2010, 2011, 
-##	 2012
+##   R package rugarch by Alexios Ghalanos Copyright (C) 2008-2013.
 ##   This file is part of the R package rugarch.
 ##
 ##   The R package rugarch is free software: you can redistribute it and/or modify
@@ -20,7 +19,7 @@
 # SECTION iGARCH fit
 #---------------------------------------------------------------------------------
 .igarchfit = function(spec, data, out.sample = 0, solver = "solnp", solver.control = list(), 
-		fit.control = list(stationarity = 1, fixed.se = 0, scale = 0))
+		fit.control = list(stationarity = 1, fixed.se = 0, scale = 0, rec.init = 'all'))
 {
 	tic = Sys.time()
 	if(is.null(solver.control$trace)) trace = 0 else trace = solver.control$trace
@@ -29,6 +28,14 @@
 	fit.control$stationarity = FALSE
 	if(is.null(fit.control$fixed.se)) fit.control$fixed.se = FALSE
 	if(is.null(fit.control$scale)) fit.control$scale = FALSE
+	if(is.null(fit.control$rec.init)) fit.control$rec.init = 'all'
+	mm = match(names(fit.control), c("stationarity", "fixed.se", "scale", "rec.init"))
+	if(any(is.na(mm))){
+		idx = which(is.na(mm))
+		enx = NULL
+		for(i in 1:length(idx)) enx = c(enx, names(fit.control)[idx[i]])
+		warning(paste(c("unidentified option(s) in fit.control:\n", enx), sep="", collapse=" "), call. = FALSE, domain = NULL)
+	}
 	# if we have external regressors in variance turn off scaling
 	if(spec@model$modelinc[15] > 0) fit.control$scale = FALSE
 	if(spec@model$modelinc[5] > 0) fit.control$scale = FALSE
@@ -47,8 +54,12 @@
 	origdata = xdata$data
 	origdates = xdata$pos
 	dformat = xdata$dformat
-	# create a temporary environment to store values (deleted at end of function)
-	tempenvir = new.env(hash = TRUE, parent = .GlobalEnv)
+	# arglist replaces the use of custom environments (resolves the problem of 
+	# non-shared environment in parallel estimation, particularly windows)
+	garchenv = new.env(hash = TRUE)
+	arglist = list()
+	arglist$garchenv <- garchenv
+	arglist$pmode = 0
 	model = spec@model
 	modelinc = model$modelinc
 	pidx = model$pidx
@@ -64,24 +75,27 @@
 	} else{
 		vexdata = NULL
 	}
-	assign("dates", dates, envir = tempenvir)
-	assign("trace", trace, envir = tempenvir)
-	assign("fit.control", fit.control, envir = tempenvir)
-	
+	arglist$dates = dates
+	arglist$trace = trace
 	m =  model$maxOrder
 	model$modeldata$T = T = length(as.numeric(data))	
 	dist = model$modeldesc$distribution
 	if(fit.control$scale) dscale = sd(data) else dscale = 1
 	zdata = data/dscale
-	assign("dscale", dscale, envir = tempenvir)
-	assign("model", model, envir = tempenvir)
+	recinit = .checkrec(fit.control$rec.init, T)
+	arglist$data = zdata
+	arglist$recinit = recinit
+	arglist$dscale = dscale
+	arglist$model = model
 	ipars = model$pars
 	# Optimization Starting Parameters Vector & Bounds
-	ipars = .garchstart(ipars, data = zdata, garchenv = tempenvir)
-	assign("ipars", ipars, envir = tempenvir)
+	tmp = .garchstart(ipars, arglist)
+	ipars = arglist$ipars = tmp$pars
+	arglist$tmph  = tmp$tmph
 	# we now split out any fixed parameters
 	estidx = as.logical( ipars[,4] )
-	assign("estidx", estidx, envir = tempenvir)
+	arglist$estidx = estidx	
+	arglist$fit.control = fit.control
 	
 	npars = sum(estidx)
 	if(any(ipars[,2]==1)){
@@ -89,18 +103,17 @@
 			if(fit.control$fixed.se==0) {
 				# if all parameters are fixed an no standard erros are to
 				# be calculated then we return a ugarchfilter object
-				cat("\nugarchfit-->warning: all parameters fixed...returning ugarchfilter 
-								object instead\n")
-				return(ugarchfilter(data = data, spec = spec))
+				cat("\nugarchfit-->warning: all parameters fixed...returning ugarchfilter object instead\n")
+				return(ugarchfilter(data = data, spec = spec, out.sample = out.sample))
 			} else{
 				# if all parameters are fixed but we require standard errors, we
 				# skip the solver
 				use.solver = 0
 				ipars[ipars[,2]==1, 4] = 1
 				ipars[ipars[,2]==1, 2] = 0
-				assign("ipars", ipars, envir = tempenvir)
+				arglist$ipars = ipars
 				estidx = as.logical( ipars[,4] )
-				assign("estidx", estidx, envir = tempenvir)
+				arglist$estidx = estidx		
 			}
 		} else{
 			# with some parameters fixed we extract them (to be rejoined at end)
@@ -111,13 +124,12 @@
 		use.solver = 1
 	}
 	# start counter
-	assign(".llh", 1, envir = tempenvir)
-	
+	assign("rugarch_llh", 1, envir = garchenv)	
 	# assisgn solver constraints (solnp directly else exterior type penalty
 	# for other solvers)
 	if(fit.control$stationarity == 1 && modelinc[15] == 0){
 		cb = .garchconbounds()
-		Ifn = .igarchcon
+		Ifn = rugarch:::.igarchcon
 		ILB = cb$LB
 		IUB = cb$UB
 		if(solver == "solnp" | solver == "gosolnp" | solver == "hybrid") fit.control$stationarity = 0
@@ -127,21 +139,22 @@
 		IUB = NULL
 	}
 	# conditions controls the non-solnp solver penalty
-	assign("fit.control", fit.control, , envir = tempenvir)
-	
+	arglist$fit.control = fit.control
+
 	if(use.solver){
 		parscale = rep(1, length = npars)
 		names(parscale) = rownames(ipars[estidx,])
 		if(modelinc[1] > 0) parscale["mu"] = abs(mean(zdata))
 		if(modelinc[7] > 0) parscale["omega"] = var(zdata)
-		solution = .garchsolver(solver, pars = ipars[estidx, 1], fun = .igarchLLH, Ifn, ILB, IUB, 
-				gr = NULL, hessian = NULL, parscale = parscale, 
-				control = solver.control, LB = ipars[estidx, 5], UB = ipars[estidx, 6], 
-				ux = NULL, ci = NULL, mu = NULL, data = zdata, returnType = "llh", garchenv = tempenvir)
+		arglist$returnType = "llh"
+		solution = .garchsolver(solver, pars = ipars[estidx, 1], fun = rugarch:::.igarchLLH, 
+				Ifn, ILB, IUB, gr = NULL, hessian = NULL, parscale = parscale, 
+				control = solver.control, LB = ipars[estidx, 5], 
+				UB = ipars[estidx, 6], ux = NULL, ci = NULL, mu = NULL, 
+				arglist)
 		sol = solution$sol
 		hess = solution$hess
 		timer = Sys.time()-tic
-		ipars = get("ipars", tempenvir)
 		if(!is.null(sol$par)) ipars[estidx, 1] = sol$par else ipars[estidx, 1] = NA
 		if(sum(ipars[,2]) == 0){
 			if(modelinc[1] > 0) ipars[pidx["mu",1]:pidx["mu",2], 1] = ipars[pidx["mu",1]:pidx["mu",2], 1] * dscale
@@ -150,9 +163,17 @@
 			}
 			ipars[pidx["omega",1],1] = ipars[pidx["omega",1],1] * dscale^2
 		}
-		assign("ipars", ipars, envir = tempenvir)
+		# make sure to add last beta to ipars
+		if(modelinc[9] == 1){
+			ipars[pidx["beta", 2], 1] = 1 - sum(ipars[pidx["alpha", 1]:pidx["alpha", 2], 1])
+		} else{
+			ipars[pidx["beta", 2], 1] = 1 - sum(ipars[pidx["alpha", 1]:pidx["alpha", 2], 1]) - sum(ipars[pidx["beta", 1]:(pidx["beta", 2]-1), 1])
+		}
+		arglist$ipars = ipars
 		convergence = sol$convergence
+		if(convergence != 0) warning("\nugarchfit-->warning: solver failer to converge.")
 	} else{
+		solution = NULL
 		hess = NULL
 		timer = Sys.time()-tic
 		convergence = 0
@@ -161,25 +182,31 @@
 	}
 	fit = list()
 	# check convergence else write message/return
+
 	ipars2 = ipars
+	
 	if(convergence == 0){
-		assign("dscale", 1, envir = tempenvir)		
+		arglist$dscale = 1
 		if(sum(ipars[,2]) > 0 && fit.control$fixed.se == 1){
 			ipars[ipars[,2]==1, 4] = 1
 			ipars[ipars[,2]==1, 2] = 0
-			assign("ipars", ipars, envir = tempenvir)
+			arglist$ipars = ipars
 			estidx = as.logical( ipars[,4] )
-			assign("estidx", estidx, envir = tempenvir)
+			arglist$estidx = estidx	
 		}
-		fit = .makefitmodel(garchmodel = "iGARCH", f = .igarchLLH, data = data,  T = T, m = m, timer = timer, 
-				convergence = convergence, message = sol$message, hess, garchenv = tempenvir)
+		arglist$data = data
+		fit = .makefitmodel(garchmodel = "iGARCH", f = .igarchLLH, T = T, m = m, 
+				timer = timer, convergence = convergence, message = sol$message, 
+				hess, arglist = arglist)
 		model$modelinc[7] = modelinc[7]
 		model$modeldata$data = origdata
 		model$modeldata$dates = origdates
 		model$modeldata$date.format = dformat
 		model$pars[, 1] = fit$ipars[,1]
+		model$pars[, 5:6] = ipars2[,5:6]
 		fit$ipars[, 4] = ipars2[, 4]
 		fit$ipars[, 2] = ipars2[, 2]
+		fit$ipars[, 5:6] = ipars2[,5:6]
 	} else{
 		fit$message = sol$message
 		fit$convergence = 1
@@ -187,37 +214,36 @@
 		model$modeldata$dates = origdates
 		model$modeldata$date.format = dformat
 	}
-	# make sure to add last beta to ipars
-	if(modelinc[9] == 1){
-		fit$ipars[pidx["beta", 2], 1] = 1 - sum(fit$ipars[pidx["alpha", 1]:pidx["alpha", 2], 1])
-	} else{
-		fit$ipars[pidx["beta", 2], 1] = 1 - sum(fit$ipars[pidx["alpha", 1]:pidx["alpha", 2], 1]) - sum(fit$ipars[pidx["beta", 1]:(pidx["beta", 2]-1), 1])
-	}
+
 	# make model list to return some usefule information which
 	# will be called by other functions (show, plot, sim etc)
 	model$n.start = n.start
-
+	fit$solver = solution
 	ans = new("uGARCHfit",
 			fit = fit,
 			model = model)
-	rm(tempenvir)
 	return(ans)
 }
 
 #---------------------------------------------------------------------------------
 # SECTION iGARCH LLH
 #---------------------------------------------------------------------------------
-.igarchLLH = function(pars, data, returnType = "llh", garchenv)
+.igarchLLH = function(pars, arglist)
 {
-	model = get("model", garchenv)
-	estidx = get("estidx", garchenv)
+	data = arglist$data
+	returnType = arglist$returnType
+	garchenv = arglist$garchenv
+	# rejoin fixed and pars
+	model = arglist$model
+	estidx = arglist$estidx
 	idx = model$pidx
-	ipars = get("ipars", garchenv)
+	ipars = arglist$ipars	
 	ipars[estidx, 1] = pars
-	trace = get("trace", garchenv)
+	trace = arglist$trace	
 	T = length(data)
-	
-	fit.control = get("fit.control", garchenv)
+	dscale = arglist$dscale	
+	recinit = arglist$recinit	
+	fit.control = arglist$fit.control
 	m = model$maxOrder
 	N = c(m,T)
 	mexdata = model$modeldata$mexdata[1:T,, drop = FALSE]
@@ -230,8 +256,8 @@
 			1 - sum(ipars[idx["alpha", 1]:idx["alpha", 2], 1]))
 	
 	dist = model$modeldesc$distno
-	hm = get("tmph", garchenv)
-	rx = .arfimaxfilter(modelinc, ipars[,1], idx, mexdata = mexdata, h = hm, data = data, N = N, garchenv)
+	hm = arglist$tmph
+	rx = .arfimaxfilter(modelinc[1:21], ipars[,1], idx, mexdata = mexdata, h = hm, data = data, N = N)
 	res = rx$res
 	zrf = rx$zrf
 	res[is.na(res) | !is.finite(res) | is.nan(res)] = 0
@@ -239,43 +265,48 @@
 	kappa = 1
 	persist = (sum(ipars[idx["alpha",1]:idx["alpha",2],1]) + sum(ipars[idx["beta",1]:idx["beta",2],1]))
 	
-	# unconditional sigma value
-	mvar = mean(res*res)
+	# recursion initialization
+	mvar = ifelse(recinit$type==1, mean(res[1:recinit$n]*res[1:recinit$n]), backcastv(res, T, recinit$n))
 	if(modelinc[15]>0) {
 		mv = sum(apply(matrix(vexdata, ncol = modelinc[15]), 2, "mean")*ipars[idx["vxreg",1]:idx["vxreg",2],1])
 	} else{
 		mv = 0
 	}
+	hEst = mvar
 	if(modelinc[7]>0){
 		ipars[idx["omega",1],1] = max(eps, ipars[idx["omega",1],1]) 
-		hEst = mvar
 	} else{
-		ipars[idx["omega",1],1] = mvar * (1 - persist) - mv
-		hEst = mvar
+		mvar2 = ifelse(!is.na(modelinc[22]), modelinc[22]/dscale, mvar)
+		ipars[idx["omega",1],1] = mvar2 * (1 - persist) - mv
 	}
+	
 	if(is.na(hEst) | !is.finite(hEst) | is.nan(hEst)) hEst = var(data) * (1 - persist)
-	assign("ipars", ipars, envir = garchenv)
-	# if we have external regressors in variance equation we cannot have
-	# stationarity checks in likelihood. Stationarity conditions not valid for
-	# solnp sover which implements them internally as constraints.
-	if(fit.control$stationarity == 1 && modelinc[15] == 0){
-		if(!is.na(persist) && persist >= 1) return(llh = get(".llh", garchenv) + 0.1*(abs(get(".llh", garchenv))))
-	}
 	if(modelinc[6]>0) mexdata = as.double(as.vector(mexdata)) else mexdata = double(1)
 	if(modelinc[15]>0) vexdata = as.double(as.vector(vexdata)) else vexdata = double(1)
+	# modelinc (1:21) since 22 is either NA or numeric and no longer needed (paased to ipars if used)
+	ans = try( .C("sgarchfilterC", model = as.integer(modelinc[1:21]), 
+					pars = as.double(ipars[,1]), idx = as.integer(idx[,1]-1), 
+					hEst = as.double(hEst), x = as.double(data), res = as.double(res), 
+					e = double(T), mexdata = mexdata, vexdata = vexdata, 
+					zrf = as.double(zrf), constm = double(T), condm = double(T), 
+					m = as.integer(m), T = as.integer(T), h = double(T), 
+					z = double(T), llh = double(1), LHT = double(T), 
+					PACKAGE = "rugarch"), silent = TRUE )
 	
-	ans = try( .C("sgarchfilterC", model = as.integer(modelinc), pars = as.double(ipars[,1]), idx = as.integer(idx[,1]-1), hEst = as.double(hEst), 
-					x = as.double(data), res = as.double(res), e = double(T), mexdata = mexdata, vexdata = vexdata, zrf = as.double(zrf),
-					constm = double(T), condm = double(T), m = as.integer(m), T = as.integer(T),
-					h = double(T), z = double(T), llh = double(1), LHT = double(T)) )
 	
 	if(inherits(ans, "try-error")){
-		assign(".csol", 1, envir = garchenv)
-		assign(".filtermessage", ans, envir = garchenv)
-		if( trace > 0 ) cat(paste("\nugarchfit-->warning: ", get(".filtermessage",garchenv),"\n", sep=""))
-		return(llh = (get(".llh", garchenv) + 0.1*(abs(get(".llh", garchenv)))))
+		if(arglist$pmode!=1){
+			assign("rugarch_csol", 1, envir = garchenv)
+			assign("rugarch_filtermessage", ans, envir = garchenv)
+			if( trace > 0 ) cat(paste("\narfimafit-->warning: ", get("rugarch_filtermessage", garchenv),"\n", sep=""))
+			return(llh = (get("rugarch_llh", garchenv) + 0.1*(abs(get("rugarch_llh", garchenv)))))
+		} else{
+			return(llh = 1e10)
+		}
 	} else{
-		assign(".csol", 0, envir = garchenv)
+		if(arglist$pmode!=1){
+			assign("rugarch_csol", 0, envir = garchenv)
+		}	
 	}
 	z = ans$z
 	h = ans$h
@@ -283,27 +314,28 @@
 	llh = ans$llh
 	
 	if(is.finite(llh) && !is.na(llh) && !is.nan(llh)){
-		assign(".llh", llh, envir = garchenv)
+		if(arglist$pmode!=1) assign("rugarch_llh", llh, envir = garchenv)
 	} else {
-		llh = (get(".llh", garchenv) + 0.1*(abs(get(".llh", garchenv))))
+		if(arglist$pmode!=1) llh = (get("rugarch_llh", garchenv) + 0.1*(abs(get("rugarch_llh", garchenv)))) else llh = 1e10
 	}
 	
+	
 	# LHT = raw scores
-	#LHT = -ans$LHT[(m):T]
+	# ? -ans$LHT[(m+1):T]
 	LHT = -ans$LHT
 	ans = switch(returnType,
 			llh = llh,
 			LHT = LHT,
-			all = list(llh = llh, h = h, epsx = epsx, z = z, kappa = kappa, LHT = LHT, persistence = persist))
+			all = list(llh = llh, h = h, epsx = epsx, z = z, kappa = kappa, 
+					LHT = LHT, persistence = persist))
 	return(ans)
 }
 
 #---------------------------------------------------------------------------------
 # SECTION iGARCH filter
 #---------------------------------------------------------------------------------
-.igarchfilter = function(spec, data, out.sample = 0, n.old = NULL)
+.igarchfilter = function(spec, data, out.sample = 0, n.old = NULL, rec.init = 'all')
 {
-	.garchenv = environment()
 	xdata = .extractdata(data)
 	data = xdata$data
 	dates = xdata$pos
@@ -314,7 +346,7 @@
 	data = origdata[1:T]
 	dates = origdates[1:T]
 	if(!is.null(n.old)) Nx = n.old else Nx = length(data)
-	
+	recinit = .checkrec(rec.init, Nx)
 	model = spec@model
 	ipars = model$pars
 	pars = unlist(model$fixed.pars)
@@ -351,19 +383,18 @@
 	kappa = 1
 	persist = (sum(ipars[idx["alpha",1]:idx["alpha",2],1]) + sum(ipars[idx["beta",1]:idx["beta",2],1]))	
 	
-	rx = .arfimaxfilter(modelinc, ipars[,1], idx, mexdata = mexdata, h = 0, data = data, N = N, .garchenv)	
+	rx = .arfimaxfilter(modelinc[1:21], ipars[,1], idx, mexdata = mexdata, h = 0, data = data, N = N)
 	res = rx$res
 	zrf = rx$zrf
 	if(!is.null(n.old)){
-		rx2 = .arfimaxfilter(modelinc, ipars[,1], idx, mexdata = mexdata[1:Nx, , drop = FALSE], h = 0, data = origdata[1:Nx], N = c(m, Nx), .garchenv)	
+		rx2 = .arfimaxfilter(modelinc[1:21], ipars[,1], idx, mexdata = mexdata[1:Nx, , drop = FALSE], h = 0, data = origdata[1:Nx], N = c(m, Nx))
 		res2 = rx2$res
 		# unconditional sigma value
-		mvar = mean(res2*res2)
+		mvar = ifelse(recinit$type==1, mean(res2[1:recinit$n]*res2[1:recinit$n]), backcastv(res2, Nx, recinit$n))
 	} else{
-		mvar = mean(res*res)
-	}
-	
-	#hEst = omega + persistence*mvar
+		mvar = ifelse(recinit$type==1, mean(res[1:recinit$n]*res[1:recinit$n]), backcastv(res, T, recinit$n))
+	}	
+	hEst = mvar
 	if(modelinc[15]>0) {
 		mv = sum(apply(matrix(vexdata, ncol = modelinc[15]), 2, "mean")*ipars[idx["vxreg",1]:idx["vxreg",2],1])
 	} else{
@@ -371,20 +402,22 @@
 	}
 	if(modelinc[7]>0){
 		ipars[idx["omega",1],1] = max(eps, ipars[idx["omega",1],1]) 
-		hEst = mvar
 	} else{
-		ipars[idx["omega",1],1] = mvar * (1 - persist) - mv
-		hEst = mvar
+		mvar2 = ifelse(!is.na(modelinc[22]), modelinc[22], mvar)
+		ipars[idx["omega",1],1] = mvar2 * (1 - persist) - mv
 	}
 	
 	if(modelinc[6]>0) mexdata = as.double(as.vector(mexdata)) else mexdata = double(1)
 	if(modelinc[15]>0) vexdata = as.double(as.vector(vexdata)) else vexdata = double(1)
 	
-	ans = try( .C("sgarchfilterC", model = as.integer(modelinc), pars = as.double(ipars[,1]), 
-					idx = as.integer(idx[,1]-1), hEst = as.double(hEst), 
-					x = as.double(data), res = as.double(res), e = double(T), mexdata = mexdata, 
-					vexdata = vexdata, zrf = as.double(zrf), constm = double(T), condm = double(T), 
-					m = as.integer(m), T = as.integer(T), h = double(T), z = double(T), llh = double(1), LHT = double(T)) )
+	ans = try( .C("sgarchfilterC", model = as.integer(modelinc[1:21]), 
+					pars = as.double(ipars[,1]), idx = as.integer(idx[,1]-1), 
+					hEst = as.double(hEst), x = as.double(data), res = as.double(res), 
+					e = double(T), mexdata = mexdata, vexdata = vexdata, 
+					zrf = as.double(zrf), constm = double(T), condm = double(T), 
+					m = as.integer(m), T = as.integer(T), h = double(T), 
+					z = double(T), llh = double(1), LHT = double(T), 
+					PACKAGE = "rugarch"), silent = TRUE )
 	
 	filter = list()
 	filter$z = ans$z
@@ -400,7 +433,6 @@
 	model$modeldata$date.format = dformat
 	model$n.start = out.sample
 	
-	rm(.garchenv)
 	sol = new("uGARCHfilter",
 			filter = filter,
 			model = model)
@@ -520,9 +552,9 @@
 	}
 	
 	if(modelinc[4]>0){
-		res = arfimaf(ipars, modelinc, idx, mu, mxfi, h, epsx, z, data, N, n.ahead)
+		res = arfimaf(ipars, modelinc[1:21], idx, mu, mxfi, h, epsx, z, data, N, n.ahead)
 	} else{
-		res = armaf(ipars, modelinc, idx, mu, mxfi, h, epsx, z, data, N, n.ahead)
+		res = armaf(ipars, modelinc[1:21], idx, mu, mxfi, h, epsx, z, data, N, n.ahead)
 	}
 	return(list(h = h[(N+1):(N+n.ahead)], x = res[(N+1):(N+n.ahead)]))
 }
@@ -667,6 +699,12 @@
 		preresiduals = NA, rseed = NA, custom.dist = list(name = NA, distfit = NA), 
 		mexsimdata = NULL, vexsimdata = NULL)
 {
+	if(fit@model$modelinc[4]>0){
+		if(n.start<fit@model$modelinc[3]){
+			warning("\nugarchsim-->warning: n.start>=MA order for arfima model...automatically setting.")
+			n.start = fit@model$modelinc[3]
+		}
+	}
 	# some checks
 	if(is.na(rseed[1])){
 		sseed = as.integer(runif(1,0,as.integer(Sys.time())))
@@ -749,15 +787,19 @@
 	sigmaSim =  matrix(0, ncol = m.sim, nrow = n.sim)
 	seriesSim = matrix(0, ncol = m.sim, nrow = n.sim)
 	residSim =  matrix(0, ncol = m.sim, nrow = n.sim)
+	z[is.na(z) | is.nan(z) | !is.finite(z)] = 0
 	
 	for(i in 1:m.sim){
 		if(is.na(preresiduals[1])) preres = as.numeric(z[1:m, i])*presigma
 		
 		res = c(preres, rep(0, n))
 		
-		ans1 = try(.C("sgarchsimC", model = as.integer(modelinc), pars = as.double(ipars[,1]), idx = as.integer(idx[,1]-1), 
+		ans1 = try(.C("sgarchsimC", model = as.integer(modelinc[1:21]), 
+						pars = as.double(ipars[,1]), idx = as.integer(idx[,1]-1), 
 						h = as.double(h), z = as.double(z[,i]), res = as.double(res),
-						vexdata = as.double(vexsim[[i]]), T = as.integer(n+m), m = as.integer(m)), silent = TRUE)
+						vexdata = as.double(vexsim[[i]]), T = as.integer(n+m), 
+						m = as.integer(m), PACKAGE = "rugarch"), silent = TRUE)
+		
 		if(inherits(ans1, "try-error")) stop("\nugarchsim-->error: error in calling C function....\n")
 		sigmaSim[,i] = ans1$h[(n.start + m + 1):(n+m)]^(1/2)
 		residSim[,i] = ans1$res[(n.start + m + 1):(n+m)]
@@ -778,10 +820,10 @@
 		
 		if(modelinc[4]>0){
 			fres = c(ans1$res[(m+1):(n+m)], if(modelinc[3]>0) rep(0, modelinc[3]) else NULL)
-			ans2 = .arfimaxsim(modelinc, ipars, idx, constm[1:n, i], fres, T = n)
-			seriesSim[,i] = tail(ans2$series, n.sim)
+			ans2 = .arfimaxsim(modelinc[1:21], ipars, idx, constm[1:n, i], fres, T = n)
+			seriesSim[,i] = head(ans2$series, n.sim)
 		} else{
-			ans2 = .armaxsim(modelinc, ipars, idx, constm[,i],  x, ans1$res, T = n + m, m)
+			ans2 = .armaxsim(modelinc[1:21], ipars, idx, constm[,i],  x, ans1$res, T = n + m, m)
 			seriesSim[,i] = ans2$x[(n.start + m + 1):(n+m)]
 		}
 	}
@@ -799,6 +841,12 @@
 		preresiduals = NA, rseed = NA, custom.dist = list(name = NA, distfit = NA), 
 		mexsimdata = NULL, vexsimdata = NULL)
 {
+	if(fit@model$modelinc[4]>0){
+		if(n.start<fit@model$modelinc[3]){
+			warning("\nugarchsim-->warning: n.start>=MA order for arfima model...automatically setting.")
+			n.start = fit@model$modelinc[3]
+		}
+	}
 	if(is.na(rseed[1])){
 		sseed = as.integer(runif(1,0,as.integer(Sys.time())))
 	} else{
@@ -879,6 +927,7 @@
 	sigmaSim =  matrix(0, ncol = m.sim, nrow = n.sim)
 	seriesSim = matrix(0, ncol = m.sim, nrow = n.sim)
 	residSim =  matrix(0, ncol = m.sim, nrow = n.sim)
+	z[is.na(z) | is.nan(z) | !is.finite(z)] = 0
 	
 	if(is.na(preresiduals[1]))	preres = matrix(tail(resids, m), nrow = m, ncol = m.sim)
 	
@@ -891,8 +940,10 @@
 	}
 	e = res * res
 	
-	ans = .Call("msgarchsim", model = as.integer(modelinc), pars = as.numeric(ipars[,1]), idx = as.integer(idx[,1]-1), 
-			h = h, z = z, res = res, e = e, vxs = vxs, N = as.integer( c(m, n) ), PACKAGE = "rugarch")
+	ans = .Call("msgarchsim", model = as.integer(modelinc[1:21]), 
+			pars = as.numeric(ipars[,1]), idx = as.integer(idx[,1]-1), 
+			h = h, z = z, res = res, e = e, vxs = vxs, N = as.integer( c(m, n) ), 
+			PACKAGE = "rugarch")
 	
 	sigmaSim = matrix(sqrt( ans$h[(n.start + m + 1):(n+m), ] ), ncol = m.sim)
 	residSim = matrix(ans$res[(n.start + m + 1):(n+m), ], ncol = m.sim)
@@ -923,13 +974,15 @@
 		#if(constant) constm[,i] = constm[,i]*(1-sum(ar))
 		for(i in 1:m.sim){
 			fres = c(ans$res[(m+1):(n+m), i], if(modelinc[3]>0) rep(0, modelinc[3]) else NULL)
-			tmp = .arfimaxsim(modelinc, ipars, idx, constm[1:n, i], fres, T = n)
-			seriesSim[,i] = tail(tmp$series, n.sim)
+			tmp = .arfimaxsim(modelinc[1:21], ipars, idx, constm[1:n, i], fres, T = n)
+			seriesSim[,i] = head(tmp$series, n.sim)
 		}
 	} else{
 		#if(constant) constm = constm * ( 1 - sum(ar) )
-		tmp = .Call("marmaxsim", model = as.integer(modelinc), pars = as.numeric(ipars[,1]), idx = as.integer(idx[,1]-1), 
-				mu = constm, x = x, res = ans$res, N = as.integer( c(m, n) ), PACKAGE = "rugarch", DUP = FALSE)
+		tmp = .Call("marmaxsim", model = as.integer(modelinc[1:21]), 
+				pars = as.numeric(ipars[,1]), idx = as.integer(idx[,1]-1), 
+				mu = constm, x = x, res = ans$res, N = as.integer( c(m, n) ), 
+				PACKAGE = "rugarch", DUP = FALSE)
 		seriesSim = matrix(tmp$x[(n.start + m + 1):(n+m), ], ncol = m.sim)
 	}
 	
@@ -970,6 +1023,12 @@
 		custom.dist = list(name = NA, distfit = NA), mexsimdata = NULL, 
 		vexsimdata = NULL)
 {
+	if(spec@model$modelinc[4]>0){
+		if(n.start<spec@model$modelinc[3]){
+			warning("\nugarchpath-->warning: n.start>=MA order for arfima model...automatically setting.")
+			n.start = spec@model$modelinc[3]
+		}
+	}
 	# some checks
 	if(is.na(rseed[1])){
 		sseed = as.integer(runif(1,0,as.integer(Sys.time())))
@@ -1069,11 +1128,14 @@
 			preres = as.numeric(z[1:m,i])*presigma
 		}
 		z[1:m, 1:m.sim] = preres[1:m]/presigma[1:m]
+		z[is.na(z) | is.nan(z) | !is.finite(z)] = 0
 		res = c(preres, rep(0, n))
-		
-		ans1 = try(.C("sgarchsimC", model = as.integer(modelinc), pars = as.double(ipars[,1]), idx = as.integer(idx[,1]-1), 
+		ans1 = try(.C("sgarchsimC", model = as.integer(modelinc[1:21]), 
+						pars = as.double(ipars[,1]), idx = as.integer(idx[,1]-1), 
 						h = as.double(h), z = as.double(z[,i]), res = as.double(res),
-						vexdata = as.double(vexsim[[i]]), T = as.integer(n+m), m = as.integer(m)), silent = TRUE)
+						vexdata = as.double(vexsim[[i]]), T = as.integer(n+m), 
+						m = as.integer(m), PACKAGE = "rugarch"), silent = TRUE)
+		
 		if(inherits(ans1, "try-error")) stop("\nugarchsim-->error: error in calling C function....\n")
 		sigmaSim[,i] = ans1$h[(n.start + m + 1):(n+m)]^(1/2)
 		residSim[,i] = ans1$res[(n.start + m + 1):(n+m)]
@@ -1093,11 +1155,11 @@
 		if(modelinc[5]>0) constm[,i] = constm[,i] + ipars[idx["archm",1]:idx["archm",2], 1]*(sqrt(ans1$h)^modelinc[5])
 		if(modelinc[4]>0){
 			fres = c(ans1$res[(m+1):(n+m)], if(modelinc[3]>0) rep(0, modelinc[3]) else NULL)
-			ans2 = .arfimaxsim(modelinc, ipars, idx, constm[1:n, i], fres, T = n)
-			seriesSim[,i] = tail(ans2$series, n.sim)
+			ans2 = .arfimaxsim(modelinc[1:21], ipars, idx, constm[1:n, i], fres, T = n)
+			seriesSim[,i] = head(ans2$series, n.sim)
 		} else{
 			#if(constant) constm[,i] = constm[,i]*(1-sum(ar))
-			ans2 = .armaxsim(modelinc, ipars, idx, constm[,i],  x, ans1$res, T = n + m, m)
+			ans2 = .armaxsim(modelinc[1:21], ipars, idx, constm[,i],  x, ans1$res, T = n + m, m)
 			seriesSim[,i] = ans2$x[(n.start + m + 1):(n+m)]
 		}
 	}
@@ -1115,6 +1177,12 @@
 		custom.dist = list(name = NA, distfit = NA), mexsimdata = NULL, 
 		vexsimdata = NULL)
 {
+	if(spec@model$modelinc[4]>0){
+		if(n.start<spec@model$modelinc[3]){
+			warning("\nugarchpath-->warning: n.start>=MA order for arfima model...automatically setting.")
+			n.start = spec@model$modelinc[3]
+		}
+	}
 	# some checks
 	if(is.na(rseed[1])){
 		sseed = as.integer(runif(1,0,as.integer(Sys.time())))
@@ -1219,6 +1287,8 @@
 		preres = matrix( preresiduals, nrow = m, ncol = m.sim )
 	}
 	z[1:m, 1:m.sim] = preres[1:m, 1:m.sim]/presigma[1:m]
+	z[is.na(z) | is.nan(z) | !is.finite(z)] = 0
+	
 	res =  rbind(preres, matrix(0, nrow = n, ncol = m.sim))
 	# we'll do the external regressors first for speed.
 	if(modelinc[15]>0){
@@ -1228,8 +1298,10 @@
 		vxs = matrix(0, nrow = m + n, ncol = m.sim)
 	}
 	e = res * res
-	ans = .Call("msgarchsim", model = as.integer(modelinc), pars = as.numeric(ipars[,1]), idx = as.integer(idx[,1]-1), 
-			h = h, z = z, res = res, e = e, vxs = vxs, N = as.integer( c(m, n) ), PACKAGE = "rugarch")
+	ans = .Call("msgarchsim", model = as.integer(modelinc[1:21]), 
+			pars = as.numeric(ipars[,1]), idx = as.integer(idx[,1]-1), 
+			h = h, z = z, res = res, e = e, vxs = vxs, N = as.integer(c(m,n)), 
+			PACKAGE = "rugarch")
 	
 	sigmaSim = matrix(sqrt( ans$h[(n.start + m + 1):(n+m), ] ), ncol = m.sim)
 	residSim = matrix(ans$res[(n.start + m + 1):(n+m), ], ncol = m.sim)
@@ -1260,13 +1332,15 @@
 		#if(constant) constm[,i] = constm[,i]*(1-sum(ar))
 		for(i in 1:m.sim){
 			fres = c(ans$res[(m+1):(n+m), i], if(modelinc[3]>0) rep(0, modelinc[3]) else NULL)
-			tmp = .arfimaxsim(modelinc, ipars, idx, constm[1:n, i], fres, T = n)
-			seriesSim[,i] = tail(tmp$series, n.sim)
+			tmp = .arfimaxsim(modelinc[1:21], ipars, idx, constm[1:n, i], fres, T = n)
+			seriesSim[,i] = head(tmp$series, n.sim)
 		}
 	} else{
 		#if(constant) constm = constm * ( 1 - sum(ar) )
-		tmp = .Call("marmaxsim", model = as.integer(modelinc), pars = as.numeric(ipars[,1]), idx = as.integer(idx[,1]-1), 
-				mu = constm, x = x, res = ans$res, N = as.integer( c(m, n) ), PACKAGE = "rugarch", DUP = FALSE)
+		tmp = .Call("marmaxsim", model = as.integer(modelinc[1:21]), 
+				pars = as.numeric(ipars[,1]), idx = as.integer(idx[,1]-1), 
+				mu = constm, x = x, res = ans$res, N = as.integer(c(m,n)), 
+				PACKAGE = "rugarch", DUP = FALSE)
 		seriesSim = matrix(tmp$x[(n.start + m + 1):(n+m), ], ncol = m.sim)
 	}
 	path = list(sigmaSim = sigmaSim, seriesSim = seriesSim, residSim = residSim)
